package Kirjalista;

import java.io.PrintStream;
import java.util.Random;
import fi.jyu.mit.ohj2.Mjonot;


/**
 * Kirja-luokka
 * - Sisältää kirjan tiedot
 * - Muuttaa merkkijonon kirjan tiedoiksi
 * - Muuttaa kirjan tiedot merkkijonoksi
 */
public class Kirja {
    private int id = 0;
    private String nimi = "";
    private String kirjoittaja  = "";
    private String kirjasarja = "";
    private int sivumaara = 0;
    
    private static int seuraavaId = 1;
    
        
    /**
     * Ottaa merkkijonon ja muuttaa sen kirjan tiedoiksi, poistaa turhat välilyönnit
     * merkkijono muotoa nimi|kirjoittaja|kirjasarja|sivumäärä
     * @param mjono Merkkijono, josta kirjan tiedot saadaan
     * <pre name="test">
     * Kirja k = new Kirja();
     * k.parse("Hobitti|J.R.R. Tolkien||336");
     * k.toString() === "Hobitti|J.R.R. Tolkien||336";
     * 
     * Kirja k2 = new Kirja();
     * k2.parse("    Hobitti  |  J.R.R. Tolkien |   |336");
     * k2.toString() === "Hobitti|J.R.R. Tolkien||336";
     * 
     * 
     * </pre>
     */
    public void parse(String mjono) {
        StringBuilder sb = new StringBuilder(mjono);
        this.nimi = Mjonot.erota(sb, '|').trim();
        this.kirjoittaja = Mjonot.erota(sb, '|').trim();
        this.kirjasarja = Mjonot.erota(sb, '|').trim();
        this.sivumaara = Mjonot.erotaInt(sb, 0);
    }
    
    
    /**
     * Palauttaa kirjan tiedot merkkijonona muodossa id|nimi|kirjoittaja|kirjasarja|sivumäärä
     */
    @Override
    public String toString() {
        return (this.id + "|" + this.nimi + "|" + this.kirjoittaja + "|" + this.kirjasarja + "|" + this.sivumaara);
    }
    
    
    /**
     * Rekisteröi kirjan antamalla sille Id:n. Kasvattaa seuraavaId:tä.
     *  <pre name="test">
     *  Kirja k = new Kirja();
     *  Kirja k2 = new Kirja();
     *  k.rekisteroi();
     *  k2.rekisteroi();
     *  k.getId() === k2.getId() - 1;
     *  </pre>
     */
    public void rekisteroi() {
        this.id = seuraavaId;
        seuraavaId++;
    }
    
    
    /**
     * Apumetodi, jolla saadaan täytettyä testiarvot kirjalle.
     */
    public void esimerkkiKirja() {
        Random rand = new Random();
        this.nimi = "Hobitti" + rand.nextInt(10);
        this.kirjoittaja = "J.R.R. Tolkien";
        this.kirjasarja = "";
        this.sivumaara = rand.nextInt(1000);
    }

    
    /**
     * @return palauttaa kirjan id:n
     */
    public int getId() {
        return this.id;
    }
    
    
    /**
     * @return Kirjan nimi
     */
    public String getNimi() {
        return this.nimi;
    }
    
    /**
     * Tulostaa kirjan tiedot
     * @param out Tietovirta, johon tulostetaan
     */
    public void tulosta(PrintStream out) {
        out.println(this.id + " " + this.nimi + " " + this.kirjoittaja + " " + this.kirjasarja + " " + this.sivumaara);
    }
    
    
    /**
     * Testausta varten
     * @param args ei käytössä
     */
    public static void main(String[] args) {
        Kirja k = new Kirja();
        k.rekisteroi();
        k.esimerkkiKirja();
        k.tulosta(System.out);
        Kirja k1 = new Kirja();
        k1.rekisteroi();
        k1.esimerkkiKirja();
        k1.tulosta(System.out);
    }
}
